@@ -7,9 +7,11 @@ Different OAuth 2.0 grant types.
 
 ----
 """
-from typing import Generic
+from typing import Generic, Optional
 from .errors import (
+    InvalidClientError,
     InvalidGrantError,
+    InvalidRedirectURIError,
     InvalidRequestError,
     InvalidScopeError,
     MismatchingStateError,
@@ -25,19 +27,23 @@ from .utils import enforce_list, enforce_str, generate_token
 class GrantTypeBase(Generic[TRequest, TStorage]):
     """Base grant type that all other grant types inherit from."""
 
-    def __init__(self, storage: TStorage, client_id: str, client_secret: str):
+    def __init__(self, storage: TStorage, client_id: str, client_secret: Optional[str]):
         self.storage = storage
         self.client_id = client_id
         self.client_secret = client_secret
+        self.scope: Optional[str] = None
 
     async def create_token_response(
         self, request: TRequest, client: Client
     ) -> TokenResponse:
         """Creates token response to reply to client."""
+        if self.scope is None:
+            raise RuntimeError("validate_request() must be called first")
+
         token = await self.storage.create_token(
             request,
             client.client_id,
-            request.post.scope,
+            self.scope,
             generate_token(42),
             generate_token(48),
         )
@@ -58,7 +64,7 @@ class GrantTypeBase(Generic[TRequest, TStorage]):
         )
 
         if not client:
-            raise InvalidRequestError[TRequest](
+            raise InvalidClientError[TRequest](
                 request=request, description="Invalid client_id parameter value."
             )
 
@@ -68,6 +74,7 @@ class GrantTypeBase(Generic[TRequest, TStorage]):
         if not client.check_scope(request.post.scope):
             raise InvalidScopeError[TRequest](request=request)
 
+        self.scope = request.post.scope
         return client
 
 
@@ -91,12 +98,12 @@ class AuthorizationCodeGrantType(GrantTypeBase[TRequest, TStorage]):
         client = await super().validate_request(request)
 
         if not request.post.redirect_uri:
-            raise InvalidRequestError[TRequest](
+            raise InvalidRedirectURIError[TRequest](
                 request=request, description="Mismatching redirect URI."
             )
 
         if not client.check_redirect_uri(request.post.redirect_uri):
-            raise InvalidRequestError[TRequest](
+            raise InvalidRedirectURIError[TRequest](
                 request=request, description="Invalid redirect URI."
             )
 
@@ -130,6 +137,7 @@ class AuthorizationCodeGrantType(GrantTypeBase[TRequest, TStorage]):
         if authorization_code.is_expired:
             raise InvalidGrantError[TRequest](request=request)
 
+        self.scope = authorization_code.scope
         return client
 
     async def create_token_response(
@@ -140,7 +148,7 @@ class AuthorizationCodeGrantType(GrantTypeBase[TRequest, TStorage]):
         await self.storage.delete_authorization_code(
             request,
             client.client_id,
-            request.post.code,  # type: ignore
+            request.post.code,
         )
 
         return token_response
@@ -246,3 +254,10 @@ class ClientCredentialsGrantType(GrantTypeBase[TRequest, TStorage]):
     access a user's resources.
     See `RFC 6749 section 4.4 <https://tools.ietf.org/html/rfc6749#section-4.4>`_.
     """
+
+    async def validate_request(self, request: TRequest) -> Client:
+        # client_credentials grant requires a client_secret
+        if self.client_secret is None:
+            raise InvalidClientError[TRequest](request)
+
+        return await super().validate_request(request)
