@@ -7,11 +7,18 @@ Response objects used throughout the project.
 
 ----
 """
+import sys
+from typing import Generic, Tuple
+
+if sys.version_info >= (3, 8):
+    from typing import get_args
+else:
+    from typing_extensions import get_args
 
-from typing import Generic, List
 from .utils import generate_token
 from .errors import (
     InvalidClientError,
+    InvalidRedirectURIError,
     InvalidRequestError,
     InvalidScopeError,
     UnsupportedResponseTypeError,
@@ -35,10 +42,12 @@ class ResponseTypeBase(Generic[TRequest, TStorage]):
         self.storage = storage
 
     async def validate_request(self, request: TRequest) -> Client:
-        code_challenge_methods: List[CodeChallengeMethod] = ["plain", "S256"]
+        code_challenge_methods: Tuple[CodeChallengeMethod, ...] = get_args(
+            CodeChallengeMethod
+        )
 
         if not request.query.client_id:
-            raise InvalidRequestError[TRequest](
+            raise InvalidClientError[TRequest](
                 request=request, description="Missing client_id parameter."
             )
 
@@ -47,17 +56,17 @@ class ResponseTypeBase(Generic[TRequest, TStorage]):
         )
 
         if not client:
-            raise InvalidRequestError[TRequest](
+            raise InvalidClientError[TRequest](
                 request=request, description="Invalid client_id parameter value."
             )
 
         if not request.query.redirect_uri:
-            raise InvalidRequestError[TRequest](
+            raise InvalidRedirectURIError[TRequest](
                 request=request, description="Mismatching redirect URI."
             )
 
         if not client.check_redirect_uri(request.query.redirect_uri):
-            raise InvalidRequestError[TRequest](
+            raise InvalidRedirectURIError[TRequest](
                 request=request, description="Invalid redirect URI."
             )
 
