@@ -16,15 +16,23 @@ Warning:
 
 ----
 """
+import sys
 from dataclasses import asdict
 from http import HTTPStatus
-from typing import Any, Dict, Generic, List, Optional, Set, Tuple, Type, Union
+from typing import Any, Dict, Generic, List, Optional, Tuple, Type, Union
+
+
+if sys.version_info >= (3, 8):
+    from typing import get_args
+else:
+    from typing_extensions import get_args
 
 from .collections import HTTPHeaderDict
 from .constances import default_headers
 from .errors import (
     InsecureTransportError,
     InvalidClientError,
+    InvalidRedirectURIError,
     InvalidRequestError,
     MethodNotAllowedError,
     TemporarilyUnavailableError,
@@ -125,7 +133,7 @@ class AuthorizationServer(Generic[TRequest, TStorage]):
             )
             raise MethodNotAllowedError[TRequest](request=request, headers=headers)
 
-    @catch_errors_and_unavailability
+    @catch_errors_and_unavailability()
     async def create_token_introspection_response(self, request: TRequest) -> Response:
         """
         Returns a response object with introspection of the passed token.
@@ -160,7 +168,7 @@ class AuthorizationServer(Generic[TRequest, TStorage]):
         self.validate_request(request, ["POST"])
         client_id, _ = self.get_client_credentials(request)
 
-        token_types: Set[TokenType] = {"access_token", "refresh_token"}
+        token_types: Tuple[TokenType, ...] = get_args(TokenType)
         token_type: TokenType = "refresh_token"
 
         access_token = None
@@ -207,19 +215,19 @@ class AuthorizationServer(Generic[TRequest, TStorage]):
 
         if client_id is None or client_secret is None:
             authorization = request.headers.get("Authorization", "")
-            headers = HTTPHeaderDict({"WWW-Authenticate": "Basic"})
 
             # Get client credentials from the Authorization header.
             try:
                 client_id, client_secret = decode_auth_headers(authorization)
             except ValueError as exc:
                 raise InvalidClientError[TRequest](
-                    request=request, headers=headers
+                    description="Invalid client_id parameter value.",
+                    request=request,
                 ) from exc
 
         return client_id, client_secret
 
-    @catch_errors_and_unavailability
+    @catch_errors_and_unavailability()
     async def create_token_response(self, request: TRequest) -> Response:
         """Endpoint to obtain an access and/or ID token by presenting an
         authorization grant or refresh token.
@@ -253,7 +261,32 @@ class AuthorizationServer(Generic[TRequest, TStorage]):
             response: An :py:class:`aioauth.responses.Response` object.
         """
         self.validate_request(request, ["POST"])
-        client_id, client_secret = self.get_client_credentials(request)
+
+        client_secret: Optional[str] = None
+
+        if request.post.grant_type == "client_credentials":
+            # client_secret is required for the client_credentials grant type
+            # https://www.oauth.com/oauth2-servers/access-tokens/client-credentials/
+            client_id, client_secret = self.get_client_credentials(request)
+        else:
+            # for other grant types, client_secret is required if the client has one:
+            # If the client type is confidential or the client was issued client credentials
+            # (or assigned other authentication requirements), the client MUST authenticate
+            # with the authorization server as described in Section 3.2.1.
+            # https://www.rfc-editor.org/rfc/rfc6749#section-4.1.3
+            try:
+                client_id, client_secret = self.get_client_credentials(request)
+            except InvalidClientError as exc:
+                # When InvalidClientError is raised here it probably means that
+                # client_secret could not be found and the basic auth header
+                # had no useful data. client_secret is optional for the password
+                # grant type, so make sure we have a client_id and try to proceed.
+                client_id = request.post.client_id
+                # client_secret must not be None. When client_secret is None,
+                # storage.get_client will not run standard checks on the client_secret
+                client_secret = request.post.client_secret or ""
+                if not client_id:
+                    raise exc
 
         if not request.post.grant_type:
             # grant_type request value is empty
@@ -290,7 +323,13 @@ class AuthorizationServer(Generic[TRequest, TStorage]):
             content=content, status_code=HTTPStatus.OK, headers=default_headers
         )
 
-    @catch_errors_and_unavailability
+    @catch_errors_and_unavailability(
+        skip_redirect_on_exc=(
+            MethodNotAllowedError,
+            InvalidClientError,
+            InvalidRedirectURIError,
+        )
+    )
     async def create_authorization_response(self, request: TRequest) -> Response:
         """
         Endpoint to interact with the resource owner and obtain an
