@@ -19,7 +19,19 @@ import logging
 import random
 import string
 from base64 import b64decode, b64encode
-from typing import Any, Callable, Coroutine, Dict, List, Optional, Set, Tuple, Union
+from http import HTTPStatus
+from typing import (
+    Any,
+    Callable,
+    Coroutine,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    Union,
+)
 from urllib.parse import quote, urlencode, urlparse, urlunsplit
 
 from .collections import HTTPHeaderDict
@@ -166,7 +178,7 @@ def encode_auth_headers(client_id: str, client_secret: str) -> HTTPHeaderDict:
 
 def decode_auth_headers(authorization: str) -> Tuple[str, str]:
     """
-    Decodes an encrypted HTTP basic authentication string.
+    Decodes an encoded HTTP basic authentication string.
     Returns a tuple of the form ``(client_id, client_secret)``, and
     raises a :py:class:`aioauth.errors.InvalidClientError` exception if nothing
     could be decoded.
@@ -213,7 +225,9 @@ def create_s256_code_challenge(code_verifier: str) -> str:
     return base64.urlsafe_b64encode(data).rstrip(b"=").decode()
 
 
-def catch_errors_and_unavailability(f) -> Callable[..., Coroutine[Any, Any, Response]]:
+def catch_errors_and_unavailability(
+    skip_redirect_on_exc: Tuple[Type[OAuth2Error], ...] = (OAuth2Error,)
+) -> Callable[..., Callable[..., Coroutine[Any, Any, Response]]]:
     """
     Decorator that adds error catching to the function passed.
 
@@ -223,30 +237,49 @@ def catch_errors_and_unavailability(f) -> Callable[..., Coroutine[Any, Any, Resp
         A callable with error catching capabilities.
     """
 
-    @functools.wraps(f)
-    async def wrapper(self, request, *args, **kwargs) -> Response:
-        error: Union[TemporarilyUnavailableError, ServerError]
+    def decorator(f) -> Callable[..., Coroutine[Any, Any, Response]]:
+        @functools.wraps(f)
+        async def wrapper(self, request, *args, **kwargs) -> Response:
+            error: Union[TemporarilyUnavailableError, ServerError]
 
-        try:
-            response = await f(self, request, *args, **kwargs)
-        except OAuth2Error as exc:
-            content = ErrorResponse(error=exc.error, description=exc.description)
-            log.debug("%s %r", exc, request)
-            return Response(
-                content=asdict(content),
-                status_code=exc.status_code,
-                headers=exc.headers,
-            )
-        except Exception:
-            error = ServerError(request=request)
-            log.exception("Exception caught while processing request.")
-            content = ErrorResponse(error=error.error, description=error.description)
-            return Response(
-                content=asdict(content),
-                status_code=error.status_code,
-                headers=error.headers,
-            )
+            try:
+                response = await f(self, request, *args, **kwargs)
+            except skip_redirect_on_exc as exc:
+                content = ErrorResponse(error=exc.error, description=exc.description)
+                log.debug("%s %r", exc, request)
+                return Response(
+                    content=asdict(content),
+                    status_code=exc.status_code,
+                    headers=exc.headers,
+                )
+            except OAuth2Error as exc:
+                log.debug("%s %r", exc, request)
+                query: Dict[str, str] = {
+                    "error": exc.error,
+                }
+                if exc.description:
+                    query["error_description"] = exc.description
+                if request.settings.ERROR_URI:
+                    query["error_uri"] = request.settings.ERROR_URI
+                location = build_uri(request.query.redirect_uri, query)
+                return Response(
+                    status_code=HTTPStatus.FOUND,
+                    headers=HTTPHeaderDict({"location": location}),
+                )
+            except Exception:
+                error = ServerError(request=request)
+                log.exception("Exception caught while processing request.")
+                content = ErrorResponse(
+                    error=error.error, description=error.description
+                )
+                return Response(
+                    content=asdict(content),
+                    status_code=error.status_code,
+                    headers=error.headers,
+                )
 
-        return response
+            return response
 
-    return wrapper
+        return wrapper
+
+    return decorator
